@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2019 - 2023 Advanced Micro Devices, Inc. All rights reserved.
+Copyright (c) 2019 - 2025 Advanced Micro Devices, Inc. All rights reserved.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -27,6 +27,11 @@ Node::~Node() {
     _node = nullptr;
 }
 
+void Node::release() {
+    _prev.clear();
+    _next.clear();
+}
+
 void Node::create(std::shared_ptr<Graph> graph) {
     if (_outputs.empty() || _inputs.empty())
         THROW("Uninitialized input/output images to the node")
@@ -38,3 +43,17 @@ void Node::create(std::shared_ptr<Graph> graph) {
 void Node::update_parameters() {
     update_node();
 }
+
+void Node::add_next(const std::shared_ptr<Node> &node) {
+    // Set graph ID to the Node, to denote the graph to which it belongs to
+    if (node->get_graph_id() < 0) node->set_graph_id(_graph_id);
+    _next.emplace_back(node);
+}
+
+void Node::add_previous(const std::shared_ptr<Node> &node) {
+    for (auto& prev_node : _prev) {
+        if (prev_node->get_graph_id() != node->get_graph_id())
+            THROW("The nodes are interdependent between 2 graphs")
+    }
+    _prev.emplace_back(node);
+}
